from Answer import Answer

DEFAULT_QUESTION_TEXT = "--question--"
DEFAULT_MAX_GRADE = 1.0
DEFAULT_GRADE = 0.0
DEFAULT_ANSWER = "--answer--"


class Question:
    def __init__(self,
                 text=DEFAULT_QUESTION_TEXT,
                 max_grade=DEFAULT_MAX_GRADE,
                 grade=DEFAULT_GRADE,
                 answer=DEFAULT_ANSWER):
        self.text = text
        self.max_grade = max_grade
        self.grade = grade
        self.answer = answer

    def to_dict(self):
        dict_obj = {
            "question": self.text,
            "max_grade": self.max_grade,
            "grade": self.grade,
            "answer": self.answer if self.answer == DEFAULT_ANSWER else self.answer.to_dict()
        }

        return dict_obj



DEFAULT_TEXT = "--text--"


class TextAnswer:
    def __init__(self, html):
        self.text = ""
        self.parse_answer(html)

    def to_dict(self):
        dict_obj = {
            "text": self.text
        }

        return dict_obj

    def parse_answer(self, html):
        text_block = html.find("input", attrs={"type": "text"})
        try:
            self.text = text_block["value"]
        except:
            self.text = DEFAULT_TEXT



from bs4 import BeautifulSoup

from .Models.ChoiceAnswer import ChoiceAnswer
from .Models.MatchingAnswer import MatchingAnswer
from .Models.TextAnswer import TextAnswer

from .Models.Question import Question
from .Models.Answer import Answer
from .Models.Answer import AnswerType


class QuizParser:
    def __init__(self):
        self.tags = {
            "div": AnswerType.CHOICE,
            "span": AnswerType.TEXT,
            "table": AnswerType.MATCHING
        }

    def get_answer(self, answer_html):
        answer_type = self.tags[answer_html.name]
        if answer_type == AnswerType.CHOICE:
            answer_content = ChoiceAnswer(answer_html)
        elif answer_type == AnswerType.TEXT:
            answer_content = TextAnswer(answer_html)
        else:
            answer_content = MatchingAnswer(answer_html)

        answer = Answer(answer_type, answer_content)

        return answer

    def get_grade(self, target_html):
        grade_str = target_html.find("div", class_="grade").text.replace(" ", "")

        if grade_str.find("Баллов") != -1 or grade_str.find("Mark ") != -1:
            grade = float(grade_str[grade_str.find(" ") + 1:grade_str.find(" ") + 5].replace(",", "."))
        else:
            grade = 0.0

        return grade

    def get_max_grade(self, target_html):
        grade_str = target_html.find("div", class_="grade").text.replace(" ", "")
        max_grade = float(grade_str[len(grade_str)-4:len(grade_str)].replace(",", "."))

        return max_grade

    def to_dict(self, questions):
        dict_obj = []
        for question in questions:
            dict_obj.append(question.to_dict())

        return dict_obj

    def parse_html(self, html_text, parse_type="all", as_dict=False):
        html_content = BeautifulSoup(html_text, "html.parser")

        targets = html_content.body.find_all("div", class_="que")

        questions = []
        for target in targets:
            question = Question()
            text = target.find("div", class_="qtext").text.replace(" ", "").replace("\n", "")
            question.text = text

            if parse_type == "all":
                grade = self.get_grade(target)
                max_grade = self.get_max_grade(target)
                ablock = target.find("div", class_="ablock")
                answer = self.get_answer(ablock.find(class_="answer"))

                question.grade = grade
                question.max_grade = max_grade
                question.answer = answer
            questions.append(question)

        if as_dict:
            return self.to_dict(questions)

        return questions



class Answer:
    def __init__(self, type, content):
        self.type = type
        self.content = content

    def to_dict(self):
        dict_obj = {
            "type": self.type,
            "content": self.content.to_dict()
        }

        return dict_obj


class AnswerType:
    CHOICE = "CHOICE"
    TEXT = "TEXT"
    MATCHING = "MATCHING"



DEFAULT_OPTION_TEXT = "--option--"
DEFAULT_MATCHING_TEXT = "--text--"


class MatchingBlock:
    def __init__(self, text=DEFAULT_MATCHING_TEXT, selected_option=DEFAULT_OPTION_TEXT):
        self.text = text
        self.selected_option = selected_option


class MatchingAnswer:
    def __init__(self, html):
        self.matchings = []
        self.options = []
        self.parse_answer(html)

    def to_dict(self):
        dict_obj = {
            "options": [],
            "matchings": {}
        }

        for option in self.options:
            dict_obj["options"].append(option)
        for matching in self.matchings:
            dict_obj["matchings"][matching.text] = matching.selected_option

        return dict_obj

    def set_matching_options(self, options_block_html):
        options_html = options_block_html.find_all("option")

        for option in options_html:
            option_text = option.text.replace(" ", "").replace("\n", "")

            self.options.append(option_text)

    def get_selected_option(self, options_block_html):
        options_html = options_block_html.find_all("option")

        for option in options_html:
            option_text = option.text.replace(" ", "").replace("\n", "")
            try:
                if option["selected"] == "selected":
                    return option_text
            except:
                pass

        return DEFAULT_OPTION_TEXT

    def parse_answer(self, html):
        matching_blocks_html = html.find_all("tr", class_=["r0", "r1"])
        for block_html in matching_blocks_html:
            matching_block = MatchingBlock()

            matching_block.text = block_html.find("td", class_="text").text\
                .replace(" ", "")\
                .replace("\n", "")

            if not len(self.options):
                self.set_matching_options(block_html)
            matching_block.selected_option = self.get_selected_option(block_html)

            self.matchings.append(matching_block)



from setuptools import setup

from MoodleQuizParser import __version__

with open("README.md", "r", encoding="utf-8") as f:
    readme = f.read()

setup(
    version=__version__,
    name='py-moodle-quiz-parser',
    description='Package for parsing moodle quiz HTML documents',
    long_description=readme,
    long_description_content_type='text/markdown',
    url='https://github.com/Kononenko-Daniil/py-moodle-quiz-parser',
    author='Daniil Kononenko',
    package_dir={'MoodleQuizParser': 'MoodleQuizParser', 'MoodleQuizParser.Models': 'MoodleQuizParser/Models'},
    packages=['MoodleQuizParser', 'MoodleQuizParser.Models'],
    install_requires=['beautifulsoup4'],
    license="MIT",
    project_urls={
        "Documentation": "https://py-moodle-quiz-parser.readthedocs.io/en/latest/",
        "Source": "https://github.com/Kononenko-Daniil/py-moodle-quiz-parser"
    }
)



DEFAULT_CHOICE_TEXT = "--text--"


class ChoiceBlock:
    def __init__(self, text=DEFAULT_CHOICE_TEXT, selected=False):
        self.text = text
        self.selected = selected


class ChoiceAnswer:
    def __init__(self, html):
        self.choices = []
        self.parse_answer(html)

    def to_dict(self):
        dict_obj = []
        for choice in self.choices:
            dict_choice = {
                "text": choice.text,
                "selected": choice.selected
            }

            dict_obj.append(dict_choice)

        return dict_obj

    def parse_answer(self, html):
        choice_blocks_html = html.find_all("div", class_=["r0", "r1"])

        for block_html in choice_blocks_html:
            choice_block = ChoiceBlock()

            check_input = block_html.find("input", attrs={"checked": "checked"})
            if check_input is not None:
                choice_block.selected = True

            choice_block.text = block_html.text.replace(" ", "").replace("\n", "")

            self.choices.append(choice_block)
